import os


def touch(fname):
    open(fname, 'a').close()


def create_tree(tree, prefix=''):
    if isinstance(tree, str):
        touch(os.path.join(prefix, tree))
    elif isinstance(tree, (list, tuple)):
        for element in tree:
            create_tree(element, prefix=prefix)
    elif isinstance(tree, dict):
        for folder, content in tree.items():
            os.makedirs(os.path.join(prefix, folder), exist_ok=True)
            create_tree(content, prefix=os.path.join(prefix, folder))
    else:
        raise ValueError('Invalid tree specification')



import jinja2

from . import templates
from . import utils


def start_project(name):
    utils.create_tree({
        '.': ['README.md', 'fabfile.py'],
        name: '__init__.py',
        'unittests': {},
        'scripts': {
            'behaviour': {},
            'analysis': {},
            'theory': {},
            'figures': {},
            'tools': {},
        },
        'target': {
            'results': {
                'behaviour': {},
                'analysis': {},
                'theory': {},
                'figures': {},
            },
        },
        'requirements': ['all.txt', 'remote.txt', 'local.txt'],
    })
    with open('README.md', 'w') as f:
        f.write(jinja2.Template(templates.readme).render(module=name))

    with open('fabfile.py', 'w') as f:
        f.write(jinja2.Template(templates.fabfile).render(module=name))

    with open('requirements/all.txt', 'w') as f:
        f.write('resco')



readme = """
# {{ module }}

This is an empty readme.
"""

fabfile = """
from fabric.api import local, cd, env
from resco import api

env.module_name = '{{ module }}'
env.working_dir = '{{ module }}-wd'
env.venv = api.RemoteVirtualEnv('{{ module }}-env',
                            dependencies=['requirements/remote.txt',
                                          'requirements/all.txt'])

run_unit_tests = api.run_unit_tests
run_script = api.run_script
update_venv = api.update_venv
ls = api.ls
fetch = api.fetch
copy_data = api.copy_data
initialize_remote = api.initialize_remote
update_venv = api.update_venv


def install_local():
    local('pip install -r requirements/all.txt -r requirements/local.txt')
"""



from fabric.api import local, put, run, cd, get, env, settings

from .rvenv import RemoteVirtualEnv

__all__ = ['RemoteVirtualEnv', 'run_unit_tests', 'run_script', 'start_script']


def run_unit_tests():
    local("python -m unittest discover -s unittests/ -p '*tests.py'")


def run_script(script_name):
    run_command('{cmd}',
                create_command(script_name),
                env.venv,
                env.module_name,
                env.working_dir)


def start_script(script_name, jobname=None):
    if jobname is None:
        jobname = env.module_name
    run_command('tmux new-session -d -s ' + jobname + ' "{cmd}"',
                create_command(script_name),
                env.venv,
                env.module_name,
                env.working_dir)


def copy_data():
    put('data', env.working_dir)


def update_venv():
    put('requirements', env.working_dir)
    with cd(env.working_dir):
        env.venv.install()


def create_wd():
    with settings(warn_only=True):
        for name in ['theory', 'analysis', 'behaviour', 'figures']:
            run('mkdir -p {}'
                .format(env.working_dir + '/target/results/' + name))


def initialize_remote():
    create_wd()
    copy_data()
    update_venv()


def run_command(template, cmd, venv, module, working_dir):
    prepare(module, working_dir)

    with cd(working_dir), venv.activate():
        run(template.format(module=module, cmd=cmd))


def create_command(script_name):
    if not script_name.startswith('scripts/'):
        script_name = 'scripts/{}'.format(script_name)
    return 'PYTHONPATH=. python {}'.format(script_name)


def prepare(module, working_dir):
    run_unit_tests()
    put(module, working_dir)
    put('scripts', working_dir)


def ls(glob_pattern='*'):
    with cd(env.working_dir):
        run('ls target/{}'.format(glob_pattern))


def fetch(glob_pattern='*'):
    with cd(env.working_dir):
        get(remote_path='target/' + glob_pattern,
            local_path='%(path)s')
